/* Any copyright is dedicated to the Public Domain.
   http://creativecommons.org/publicdomain/zero/1.0/ */

.viewContainer {
  font-size: inherit; /* inherit browser's default monospace font size */
}

.rulerLines {
  background: -moz-Dialog;
  color: -moz-DialogText;
  min-width: 1.4em;
  padding-left: 4px;
  padding-right: 4px;
  text-align: end;
}

